#version 330

in vec3 worldspacePos;//uniform vec3 cameraPosition;
in vec3 cameraspaceNorm;
in vec3 cameraspaceEyeDir;
in vec3 cameraspaceLightDir;

uniform vec3 worldspaceLight;
uniform sampler2D color_Texture;//uniform sampler2D materialTex;

in vec3 worldNorm;
in vec2 fUVs;
in float object; 

out vec4 color;

uniform vec4 lightPositionOnScreen;
uniform mat4 modelMat;

uniform float materialShininess;
uniform vec3 materialSpecularColor;
uniform vec3 lightIntensity;
uniform float lightAttenuation;
uniform float ambientCoefficient;
uniform float coneAngle;
uniform vec3 coneDirection;

void main() {

    vec3 normal = normalize(transpose(inverse(mat3(modelMat))) * worldNorm);
    vec3 surfacePos = vec3(modelMat * vec4(worldspacePos, 1));
    vec4 surfaceColor = texture(color_Texture, fUVs);
    vec3 surfaceToCamera = normalize(worldspacePos - surfacePos);

    float attenuation = 0.0;

     vec3 surfaceToLight = normalize(lightPositionOnScreen.xyz - surfacePos);
     float distanceToLight = length(lightPositionOnScreen.xyz - surfacePos);
	 
	// float fallOff = dot(surfaceToLight, coneDirection);

	 attenuation = 1.0 / (1.0 + lightAttenuation * pow(distanceToLight, 2));

	float lightToSurfaceAngle = degrees(acos(dot(-surfaceToLight, normalize(coneDirection))));
        if(lightToSurfaceAngle > coneAngle){
            attenuation = 0.0; 
    }
	else{
		attenuation = (1-(1-lightToSurfaceAngle)*1/(1-coneAngle));
	}

    //ambient
    vec3 ambient = ambientCoefficient * surfaceColor.rgb * lightIntensity;

    //diffuse
    float diffuseCoefficient = max(0.0, dot(normal, surfaceToLight));
    vec3 diffuse = diffuseCoefficient * surfaceColor.rgb * lightIntensity;
    
    //specular
    float specularCoefficient = 0.0;
    if(diffuseCoefficient > 0.0)
        {specularCoefficient = pow(max(0.0, dot(surfaceToCamera, reflect(-surfaceToLight, normal))), materialShininess);}

    vec3 specular = specularCoefficient * materialSpecularColor * lightIntensity;

    color = vec4(pow(ambient + attenuation*(diffuse + specular), vec3(1.0/2.2)), surfaceColor.a);
}